@@ -44,7 +44,7 @@ class MessageSpec(object):
         self.length = length
    
         # Attributes that can be set on the object
-        self.valid_attributes = set(self.arguments) | {'time'}
+        self.settable_attributes = set(self.arguments) | {'time'}
 
     def signature(self):
         """Return call signature for Message constructor for this type.
@@ -183,14 +183,14 @@ def check_pitch(pitch):
 def check_data(data_bytes):
     """Check type of data_byte and type and range of each data byte.
 
-    Returns the data bytes as a tuple of integers.
+    Returns the data bytes as a SysexData object.
 
     Raises TypeError if value is not iterable.
     Raises TypeError if one of the bytes is not an integer.
     Raises ValueError if one of the bytes is out of range 0..127.
     """
     # Make the sequence immutable.
-    data_bytes = tuple(data_bytes)
+    data_bytes = SysexData(data_bytes)
 
     for byte in data_bytes:
         check_databyte(byte)
@@ -198,6 +198,30 @@ def check_data(data_bytes):
     return data_bytes
 
 
+def check_frame_type(value):
+    """Check type and value SMPTE quarter frame type.
+
+    Raises TypeError if the value is not an integer.
+    Raises ValueError if the value is out of range.
+    """
+    if not isinstance(value, int):
+        raise TypeError('frame_type must be an integer')
+    elif not 0 <= value <= 7:
+        raise ValueError('frame_type must be in range 0..7')
+
+
+def check_frame_value(value):
+    """Check type and value of SMPTE quarter frame value.
+
+    Raises TypeError if the value is not an integer.
+    Raises ValueError if the value is out of range.
+    """
+    if not isinstance(value, int):
+        raise TypeError('frame_value must be an integer')
+    elif not 0 <= value <= 15:
+        raise ValueError('frame_value must be in range 0..15')
+
+
 def check_databyte(value):
     """Raise exception of byte has wrong type or is out of range
 
@@ -241,67 +265,6 @@ class BaseMessage(object):
 
     Can be subclassed to create meta messages, for example.
     """
-    pass
-
-class Message(BaseMessage):
-    """
-    MIDI message class.
-    """
-
-    # Quick lookup of specs by name or status_byte.
-    _spec_lookup = build_spec_lookup(get_message_specs())
-
-    def __init__(self, type_, **parameters):
-        """Create a new message.
-
-        The first argument is typically the type of message to create,
-        for example 'note_on'.
-
-        It can also be the status_byte, that is the first byte of the
-        message. For channel messages, the channel (lower 4 bits of
-        the status_byte) is masked out from the lower 4 bits of the
-        status byte. This can be overriden by passing the 'channel'
-        keyword argument.
-        """
-        try:
-            spec = self._spec_lookup[type_]
-        except KeyError:
-            text = '{!r} is an invalid type name or status byte'
-            raise ValueError(text.format(type_))
-
-        self._set('_spec', spec)
-        self._set('type', self._spec.type)
-
-        self._set_attributes_to_default_values(type_)
-        self._override_attributes(parameters)
-
-    def _set_attributes_to_default_values(self, type_):
-        for name in self._spec.arguments:
-            if name == 'velocity':
-                self._set('velocity', 0x40)
-            elif name == 'channel':
-                # This is a channel message, so if the first
-                # argument to this function was a status_byte,
-                # the lower 4 bits will contain the channel.
-                if isinstance(type_, int):
-                    self._set('channel', type_ & 0x0f)
-                else:
-                    self._set('channel', 0)
-            elif name == 'data':
-                self._set('data', ())
-            else:
-                self._set(name, 0)
-        self._set('time', 0)
-
-    def _override_attributes(self, parameters):
-        for name, value in parameters.items():
-            try:
-                setattr(self, name, value)
-            except AttributeError:
-                raise ValueError('{!r} is an invalid'
-                                 ' keyword argument for this message type'
-                                 ''.format(name))
-
     def copy(self, **overrides):
         """Return a copy of the message.
 
@@ -314,68 +277,23 @@ class Message(BaseMessage):
             a = Message('note_on')
             b = a.copy(velocity=32)
         """
+        for name in overrides:
+            if name not in self._spec.settable_attributes:
+                text = '{!r} is an invalid argument for this message type'
+                raise ValueError(text.format(name))
+
         # Get values from this object
         arguments = {}
-        for name in self._spec.valid_attributes:
+        for name in self._spec.settable_attributes:
             if name in overrides:
                 arguments[name] = overrides[name]
             else:
                 arguments[name] = getattr(self, name)
 
-        for name in overrides:
-            if name not in self._spec.valid_attributes:
-                text = '{!r} is an invalid argument for this message type'
-                raise ValueError(text.format(name))
-
         return self.__class__(self.type, **arguments)
 
-    def _set(self, name, value):
-        """Sets an attribute directly, bypassing all type and value checks"""
-        self.__dict__[name] = value
-
-    def __setattr__(self, name, value):
-        """Set an attribute."""
-
-        if name in self._spec.valid_attributes:
-            try:
-                check = globals()['check_{}'.format(name)]
-            except KeyError:
-                check = check_databyte
-
-            ret = check(value)
-            if name == 'data':
-                value = ret
-
-            self.__dict__[name] = value
-        elif name in self.__dict__:
-            raise AttributeError('{} attribute is read only'.format(name))
-        else:
-            raise AttributeError('{} message has no attribute {}'.format(
-                                 self.type, name))
-
-    def __delattr__(self, name):
-        raise AttributeError('attribute can not be deleted')
-
     def bytes(self):
-        """Encode message and return as a list of integers."""
-
-        status_byte = self._spec.status_byte
-        if status_byte < 0xf0:
-            # Add channel (lower 4 bits) to status byte.
-            # Those bits in spec.status_byte are always 0.
-            status_byte |= self.channel
-
-        message_bytes = [status_byte]
-
-        for name in self._spec.arguments:
-            value = getattr(self, name)
-            try:
-                encode = globals()['encode_{}'.format(name)]
-                message_bytes.extend(encode(value))
-            except KeyError:
-                message_bytes.append(value)
-
-        return message_bytes
+        raise ValueError('bytes() is not implemented in this class')
 
     def bin(self):
         """Encode message and return as a bytearray.
@@ -391,31 +309,116 @@ class Message(BaseMessage):
         """
         return sep.join(['{:02X}'.format(byte) for byte in self.bytes()])
 
+    def __eq__(self, other):
+        """Compare message to another for equality.
+        
+        Key for comparison: (msg.type, msg.channel, msg.note, msg.velocity).
+        """
+        if not isinstance(other, BaseMessage):
+            raise TypeError('comparison between message and another type')
+
+        return self.bytes() == other.bytes()
+
+class SysexData(tuple):
+    """Special kind of tuple accepts and converts any sequence in +=."""
+    def __iadd__(self, other):
+        return SysexData(self + check_data(other))
+
+class Message(BaseMessage):
+    """
+    MIDI message class.
+    """
+
+    # Quick lookup of specs by name or status_byte.
+    _spec_lookup = build_spec_lookup(get_message_specs())
+
+    # This is needed for __init__() so it doesn't accept status bytes.
+    _type_lookup = {name: value for name, value in _spec_lookup.items() \
+                        if not isinstance(name, int)}
+
+    def __init__(self, type, **arguments):
+        """Create a new message.
+
+        The first argument is typically the type of message to create,
+        for example 'note_on'.
+        """
+        try:
+            spec = self._type_lookup[type]
+        except KeyError:
+            raise ValueError('invalid message type {!r}'.format(type))
+
+        self.__dict__['type'] = type
+        self.__dict__['_spec'] = spec
+
+        # Set default values.
+        for name in spec.arguments:
+            if name == 'velocity':
+                self.__dict__['velocity'] = 0x40
+            elif name == 'data':
+                self.__dict__['data'] = SysexData()
+            else:
+                self.__dict__[name] = 0
+        self.__dict__['time'] = 0
+
+        # Override defaults.
+        for name, value in arguments.items():
+            setattr(self, name, value)
+
+    def __setattr__(self, name, value):
+        """Set an attribute."""
+        if name in self._spec.settable_attributes:
+            try:
+                if name == 'data':
+                    value = check_data(value)
+                else:
+                    globals()['check_{}'.format(name)](value)
+            except KeyError:
+                check_databyte(value)
+
+            self.__dict__[name] = value
+        elif name in self.__dict__:
+            raise AttributeError('{} attribute is read only'.format(name))
+        else:
+            raise AttributeError(
+                '{} message has no attribute {}'.format(self.type, name))
+
+    def __delattr__(self, name):
+        raise AttributeError('attribute can not be deleted')
+
+    def bytes(self):
+        """Encode message and return as a list of integers."""
+
+        status_byte = self._spec.status_byte
+        if status_byte < 0xf0:
+            # Add channel (lower 4 bits) to status byte.
+            # Those bits in spec.status_byte are always 0.
+            status_byte |= self.channel
+
+        message_bytes = [status_byte]
+
+        if self.type == 'quarter_frame':
+            message_bytes.append(self.frame_type << 4 | self.frame_value)
+        else:
+            for name in self._spec.arguments:
+                value = getattr(self, name)
+                try:
+                    encode = globals()['encode_{}'.format(name)]
+                    message_bytes.extend(encode(value))
+                except KeyError:
+                    message_bytes.append(value)
+
+        return message_bytes
+
     def __repr__(self):
         parts = []
 
         for name in self._spec.arguments + ('time',):
             parts.append('{}={!r}'.format(name, getattr(self, name)))
 
-        return '<message {} {}>'.format(self.type, ', '.join(parts))
+        return '<message {} {}>'.format(self.type, ' '.join(parts))
 
     def __str__(self):
-        return _format_as_string(self)
-
-    def __eq__(self, other):
-        """Compare message to another for equality.
-        
-        Key for comparison: (msg.type, msg.channel, msg.note, msg.velocity).
-        """
-        if not isinstance(other, Message):
-            raise TypeError('comparison between Message and another type')
-
-        def key(msg):
-            """Return a key for comparison."""
-            return [msg.type] + [
-                getattr(msg, arg) for arg in msg._spec.arguments]
-
-        return key(self) == key(other)
+        return format_as_string(self)
 
     def __len__(self):
         if self.type == 'sysex':
@@ -568,10 +571,12 @@ def parse_string(text):
 def parse_string_stream(stream):
     """Parse a stram of messages and yield (message, error_message)
 
-    stream can be any iterable that generates text strings. If
-    a line can be parsed, (message, None) is returned. If it can't
-    be parsed (None, error_message) is returned. The error message
-    containes the line number where the error occured.
+    stream can be any iterable that generates text strings, where each
+    string is a string encoded message.
+
+    If a string can be parsed, (message, None) is returned. If it
+    can't be parsed (None, error_message) is returned. The error
+    message containes the line number where the error occurred.
     """
     line_number = 1
     for line in stream:
@@ -587,11 +592,12 @@ def parse_string_stream(stream):
         line_number += 1
 
 
-def _format_as_string(message):
+def format_as_string(message, include_time=True):
     """Format a message and return as a string.
 
-    There is no reason to call this function directly.
-    Use str(message) instead.
+    This is equivalent to str(message).
+
+    To leave out the time attribute, pass include_time=False.
     """
     if not isinstance(message, Message):
         raise ValueError('message must be a mido.Message object')
@@ -599,7 +605,11 @@ def _format_as_string(message):
     words = []
     words.append(message.type)
 
-    for name in message._spec.arguments + ('time',):
+    names = message._spec.arguments
+    if include_time:
+        names += ('time',)
+
+    for name in names:
         value = getattr(message, name)
         if name == 'data':
             value = '({})'.format(','.join([str(byte) for byte in value]))
